#!/usr/bin/perl
#
#    Webmin module for Exim
#    Copyright (C) 2001 by Alexandre MATHIEU <mtlx@free.fr>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# Much of the here is directly borrowed from the 'eximstat' perl script
# coming with the standard 'exim' tarball.
#
# dspl_stat.cgi
# Display statistics

require 5;

use strict;
use integer;
use vars qw(%config %text %in @logs @time_units @size_units);

my %msgs;
my %users;
my %hosts;
my %transports;
my %errors;
my %relayed;

sub format_size
  {
    my $size = shift;
    foreach my $unit (@size_units)
      {
	return sprintf('%4d %s', $size, $unit) if $size < (1<<10);
	$size >>= 10;
      }
  }

require './exim-lib.pl';

ReadParse();

my $logfile = get_log_directory ();
$logfile =~ s/%s/main/;

my $begin = '9999-99-99 99:99:99';
my $end = '0000-00-00 00:00:00';
if (exists $in{ti_v} and exists $in{ti_u})
  {
    my $time_interval_value = parse_CGI ($in{ti_v}, 8, '0-9');
    my $time_interval_unit = parse_CGI ($in{ti_u}, 1, 'a-z', @time_units);

    if (defined $time_interval_value and defined $time_interval_unit)
      {
	my ($sec, $min , $hour, $mday, $mon,$year,$wday,$yday,$isdst) = localtime(time - $config{time_interval}*60);

	$begin = sprintf('%04u-%02u-%02u %02u:%02u%02u', 1900+$year, $mon, $mday, $hour, $min, $sec);
      }
  }

my ($received_count_total, $delivered_count_total, $delayed_count_total, $errors_count_total);
my ($received_data_total, $delivered_data_total);
my $relayed_unshown;

# Let's go ! Process the log file.

open LOG, $logfile or die "Unable to open $logfile";

while (<LOG>)
  {
    next if length $_ < 38;
    next unless /^\d{4}\-\d{2}\-\d{2}\s\d{2}:\d{2}:\d{2}/;

    my $time = substr($_, 0, 19);

    $begin = $time if $time lt $begin;
    $end = $time if $time gt $end;

    my $id = substr($_, 20, 16);

    my $flag = substr($_, 37, 2);

    my $refmsg = $msgs{$id};

  FLAG: {
      $flag eq '<=' and do
	{                                           # We receive a message.
	  my ($size) = /\sS=(\d+)/;
	  $size = 0 unless defined $size;
	  $msgs{$id}{size} = $size;
	  $msgs{$id}{received_time} = $time;

	  my ($host) = /\sH=(\S+)/;
	  if (defined $host)
	    {
	      if ($config{stat_show_relay})
		{
		  my ($from) = /^.{40}(\S+)/;
		  my ($ip) = /\sH=\S+(?:(?=\s\()\s\S+)?(\s\[[^]]*\])/;
		  $ip = '' unless defined $ip;
		  $msgs{$id}{from_host} = $host;
		  $msgs{$id}{from_ip} = $ip;
		  $msgs{$id}{from_address} = $from;
		}
	    }
	  else
	    {
	      $host = 'local';
	      my ($user) = /\sU=(\S+)/;
	      if (defined $user)
		{
		  $users{$user}{received_count}++;
		  $users{$user}{received_data} = 0 unless exists $users{$user}{received_data};
		  $users{$user}{received_data} += $size;
		}
	    }

	  $hosts{$host}{received_count}++;
	  $hosts{$host}{received_data} = 0 unless exists $hosts{$host}{received_data};
	  $hosts{$host}{received_data} += $size;

	  $received_count_total++;
	  $received_data_total += $size;

	  last FLAG;
	};

      $flag eq '=>' && do
	{                                    # We deliver a message.
	  my $size = $msgs{$id}{size};
	  $msgs{$id}{delivered_time} = $time;
	  $size = $msgs{$id}{size} = 0 unless defined $size;
	  my ($host) = /\sH=(\S+)/;

	  if (defined $host)
	    {
	      $msgs{$id}{remote_delivered} = 0 unless defined($msgs{$id}{remote_delivered});
	      $msgs{$id}{remote_delivered}++;

	      # Determine relaying address if either only one address listed,
	      # or two the same. If they are different, it implies a forwarding
	      # or aliasing, which is not relaying. Note that for multi-aliased
	      # addresses, there may be a further address between the first
	      # and last.

	      if ($config{stat_show_relay} and defined $msgs{$id}{from_host})
		{
		  my $old = my $new = '';
		  my ($ip) = /\sH=\S+(?:(?=\s\()\s\S+)?(\s\[[^]]*\])/;
		  $ip = '' unless defined $ip;
		  if (/^.{40}(\S+)(?:\s+\([^)]\))?\s+<([^>]+)>/)
		  { $old = $1; $new = $2 }
		  else
		    { $old = $new = '' }

		  if ("\L$new" eq "\L$old")
		    {
		      ($old) = /^.{40}(\S+)/ if $old eq '';
		      my $key = "H=\L$msgs{$id}{from_host}\E A=\L$msgs{$id}{from_address}\E => " .
			"H=\L$host\E$ip A=\L$old\E";
		      if (!defined $config{stat_relay_pattern} or $key !~ /$config{stat_relay_pattern}/o)
			{
			  $relayed{$key} = 0 if !defined $relayed{$key};
			  $relayed{$key}++;
			}
		      else { $relayed_unshown++ }
		    }
		}
	    }
	  else
	    {
	      $host = 'local';
	      my ($rest) = substr($_,40);
	      if (my ($user) = split(($rest =~ /</)? ' <' : ' ', $rest))
		{
		  if ($user =~ /^[\/|]/)
		    {
		      my ($parent) = $_ =~ /(<[^@]+@?[^>]*>)/;
		      $user = "$user $parent" if defined $parent;
		    }
		  $users{$user}{delivered_count}++;
		  $users{$user}{delivered_data} = 0 unless defined $users{$user}{delivered_data};
		  $users{$user}{delivered_data} += $size
		}
	    }

	  $hosts{$host}{delivered_count}++;
	  $hosts{$host}{delivered_data} = 0 unless defined $hosts{$host}{delivered_data};
	  $hosts{$host}{delivered_data} += $size;
	
	  if (!$config{stat_show_transport})
	    {
	      my ($transport) = /\sT=(\S+)/;
	      $transport = ":blackhole:" unless defined $transport;
	      $transports{$transport}{data} = 0 unless defined $transports{$transport}{data};
	      $transports{$transport}{count}++;
	      $transports{$transport}{data} += $size
	    }
	  $delivered_count_total++;
	  $delivered_data_total += $size;
	
	  last FLAG
	};

      $flag eq '==' and defined $msgs{$id}{size} and not defined $msgs{$id}{delayed} and do
	{
	  $msgs{$id}{delayed} = 1;
	  $delayed_count_total++;
	  last FLAG
	};

    $flag eq '**' && do                           # We've got an error !
	{
	  $msgs{$id}{had_error} = 1 if defined $msgs{$id}{size};
	  if ($config{stat_show_errors})
	    {
	      my $error = substr($_, 40);
	      $errors{$error}++
	    }
	  $errors_count_total++;
	  last FLAG
	};

    /Completed$/ and do                          # Ok, the message leaves the queue.

	{
	  my $arrival = $msgs{$id}{received_time};

	  my $queued = seconds ($time);
	  if (defined $arrival)
	    {
	      $queued -= seconds ($arrival)
	    }
	  else
	    {
	      my ($base62_time) = $id =~ /^\d{6}/;
	      $queued -= base62_to_decimal($base62_time)
	    }
	  $msgs{$id}{queued} = $queued;

	  if ($config{stat_show_relay})
	    {
	      delete $msgs{$id}{from_host};
	      delete $msgs{$id}{from_address}
	    }
	  $msgs{$id}{completed_time} = $time;
	}
      }
  }
close LOG;

# End of log file processing.

my $failed_count = 0;
my $delivered_hosts = 0;
my $received_hosts = 0;

foreach my $id (keys %msgs)
  {
    my $refmsg = $msgs{$id};
    $failed_count++ if exists $$refmsg{had_error};
  }

foreach my $host (keys %hosts)
  {
    my $refhost = $hosts{$host};
    $received_hosts++ if exists $$refhost{received_data};
    $delivered_hosts++ if exists $$refhost{delivered_data};
  }

# Here we display some statistics.

# Webmin Header.
header($text{stat_title}, '');
print "<hr/>\n";

#print 'log file: ', $logfile, '<br>';
print "<b>$text{stat_begin} </b>", $begin, '&nbsp;';
print "<b>$text{stat_end} </b>", $end, '.<br/>';

# First the summary.

print <<HTML
<p><b>$text{stat_summary}</b></p>
<table border>
  <tr $::tb>
    <th>$text{stat_total}</th>
    <th>$text{stat_volume}</th>
    <th>$text{stat_messages}</th>
    <th>$text{stat_host}</th>
    <th>$text{stat_delayed}</th>
    <th>$text{stat_failed}</th>
  </tr>
  <tr $::cb>
    <th $::tb>$text{stat_received}</th>
HTML
;
{
  no integer;
  print_td (format_size ($received_data_total));
  print_td ($received_count_total);
  print_td ($received_hosts);
  print '<td>', $delayed_count_total, '&nbsp;(', sprintf('%4.1f%%', ($received_count_total == 0)?0:($delayed_count_total*100)/$received_count_total), ')</td>';
  print '<td>', $failed_count, '&nbsp;(', sprintf ('%4.1f%%', ($received_count_total == 0)?0:($failed_count*100)/$received_count_total), ')</td>';
}
print '</tr>', "\n";

print "<tr $::cb>\n";
print "<th $::tb>$text{stat_delivered}</th>\n";
print_td (format_size ($delivered_data_total));
print_td ($delivered_count_total);
print_td ($delivered_hosts);
print '</tr>', "\n";
print '</table>', "\n";

# Then the totals by transport if required.

if ($config{stat_show_transport})
  {
    print '<p><b>', $text{stat_deliveries_by_transport}, '</b></p>', "\n";
    print <<HTML
<table border>
  <tr $::tb>
    <th>$text{stat_transport}</th>
    <th>$text{stat_volume}</th>
    <th>$text{stat_messages}</th>
  </tr>
HTML
      ;
    foreach my $transport (sort keys %transports)
      {
	print_tr ($transport,
		  format_size ($transports{$transport}{data}),
		  $transports{$transport}{count});
      }
    print '</table>';
  }

# Print relay information if required

if ($config{stat_show_relay})
  {
    if (scalar(keys %relayed) > 0 or $relayed_unshown > 0)
      {
	my $shown = 0;
	print '<p><b>', $text{stat_relayed}, '</b></p>', "\n";
	print '<table border>', "\n";
	print "<tr $::tb>\n";
	print '<th>', $text{stat_messages}, '</th><th>', $text{stat_from}, '</th><th>', $text{stat_to}, '</th>', "\n";
	print '</tr>', "\n";

	foreach my $key (sort keys %relayed)
	  {
	    my $count = $relayed{$key};
	    $shown += $count;
	    $key =~ s/[HA]=//g;
	    my ($from,$to) = split(/=> /, $key);
	    print_tr ($count, $from, $to);
	  }
	print '</table>', "\n";
	print "<p><b>Total: $shown (plus $relayed_unshown unshown)</b></p>\n";
      }
    else
      {
	print '<p><b>', $text{stat_no_relayed_messages},'</b></p>', "\n";
      }
  }

# If the topcount is zero, print no league tables

if ($config{stat_topcount} > 0)
  {
    print_league_table("sending host", 'host', 'received');
    print_league_table("local sender", 'user', 'received')
      if $config{stat_local_league_table};
    print_league_table("destination", 'host', 'delivered');
    print_league_table("local destination", 'user', 'delivered')
      if $config{stat_local_league_table};
  }

# Omit error statistics if configured out

if ($config{stat_show_errors})
  {
    if (scalar(keys %errors) != 0)
      {
	print '<p><b>', $text{stat_list_of_errors}, ':</b></p>', "\n";
	print '<ul>', "\n";
	foreach my $error (sort keys %errors)
	  {
	    chop(my $text = $error);
	    print '<li>', $errors{$error}, '&nbsp;-&gt;&nbsp;', $text, '</li>', "\n";
	  }
	print '</ul>', "\n";
      }
    if ($errors_count_total == 0)
      {
	print '<p><b>', $text{stat_no_errors}, '</b></p>'
      }
    elsif ($errors_count_total == 1)
      {
	print '<p><b>', $text{stat_one_error}, '</b></p>'
      }
    else
      {
	print '<p><b>', &text('stat_errors', $errors_count_total), '</b></p>'
      }
    print "\n";
  }
# Footer
print '<hr/>', "\n";
footer ('', $text{index_title});
1;

sub print_league_table {
  my $text = shift;
  my $type = shift;    # 'user' or 'host'
  my $action = shift;  # 'delivered' or 'received'

  my $count = $action.'_count';
  my $data = $action.'_data';
  my $refhash;
  if ($type eq 'user') { $refhash = \%users }
  elsif ($type eq 'host') { $refhash = \%hosts }
  else { return undef }

  my $name = ($config{stat_topcount} == 1)? "$text" : "$config{stat_topcount} ${text}s";

  print "<p><b>Top $name by message count</b></p>\n";
  print '<table border="1">', "\n";
  print_th ($text{stat_messages}, $text{stat_volume}, ucfirst $text{"stat_$type"});
  my $n = 1;
  foreach my $key (sort
		   {
		     $$refhash{$b}{$count} <=> $$refhash{$a}{$count}
		       or $$refhash{$b}{$data}  <=> $$refhash{$a}{$data}
			 or $a cmp $b
		    }
		   keys %{$refhash})
    {
      if ($$refhash{$key}{$count})
	{
	  print_tr (sprintf ('%7d', $$refhash{$key}{$count}), format_size ($$refhash{$key}{$data}), $key);
	}
      last if $n++ >= $config{stat_topcount};
    }
  print '</table>', "\n";

  print "<p><b>Top $name by volume</b></p>\n";
  print '<table border="1">', "\n";
  print_th ($text{stat_messages}, $text{stat_volume}, ucfirst $text{"stat_$type"});
  $n = 1;
  foreach my $key (sort
		{
		  $$refhash{$b}{$data}  <=> $$refhash{$a}{$data}
		    or $$refhash{$b}{$count} <=> $$refhash{$a}{$count}
		      or $a cmp $b
		    }
		keys %{$refhash})
    {
      if ( $$refhash{$key}{$count})
	{
	  print_tr (sprintf ('%7d', $$refhash{$key}{$count}), format_size ($$refhash{$key}{$data}), $key);
	}
      last if $n++ >= $config{stat_topcount};
    }
  print '</table>', "\n";
  print "\n";
}
