#!/usr/bin/perl
#
#    Webmin module for Exim
#    Copyright (C) 2001 by Alexandre MATHIEU <mtlx@free.fr>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#
# exim-lib.pl
# Some subs needed by the Exim module

require 5;

use strict;  # Be strict !
use integer; # Be fast !
use vars qw(@time_units @size_units %config %text $exim_path);

do '../web-lib.pl'; # Required by Webmin

init_config();      # Initiate the Webmin stuff

@time_units = ('m', 'h', 'd', 'w');
@size_units = ('B', 'K', 'M', 'G');

$exim_path = $config{exim_path}?$config{exim_path}:'exim';
my $exim_conf = defined $config{exim_conf}?$config{exim_conf}:'';

# Cast exim and return a descriptor or undef if something wierd happens.
# &cast_exim ('config_file', 'command', 'redirection')
sub cast_exim
{
    push my @command, $exim_path;
    my $config_file  = shift;
    if (defined $config_file) {	push @command, '-C'.$config_file }
    elsif ($exim_conf) { push @command, '-C'.$exim_conf }
    push @command, shift;
    my $redirect = shift;
    push @command, defined $redirect?$redirect:'2>/dev/null';
    open EXIM, join ' ',@command,'|' or return undef;
    return \*EXIM
}

sub display_error_page
{
    header($text{error}, '', undef, 1, 1);
    print "<hr/>\n";
    print '<p>';
    print text(@_);
    print '</p>', "\n";
    print '<br/>', "\n";
    print '<hr/>', "\n";
    footer('/', $text{index});
}

# Return the default configuration file.
# &get_config_file_path()
sub get_config_file_path
{
    my $exim = &cast_exim (undef, '-bP configure_file') or return undef;
    my $config_file = <$exim>;
    chomp $config_file;
    close $exim or return undef;
    return $config_file
}

# Check if the given configuration exist (and is readable?)
# &check_config_file ('path_to_config_file')
sub check_config_file
{
    my $exim = &cast_exim (shift, '-bP configure_file', '2>&1 1>/dev/null')
	or return undef;
    my @error_msg = <$exim>;
    return undef if close $exim;
    return @error_msg
}

# Return the path to the exim spool directory
# &get_spool_directory()
sub get_spool_directory
{
    my $exim = &cast_exim (undef, '-bP spool_directory') or return undef;
    $_ = <$exim>;
    chomp;
    close $exim or return undef;
    my @spool_directory = split_line($_);
    return pop @spool_directory
}

# Return a string as the path to the exim log files
# where '%s' must be replace by 'main', 'panic' or 'reject'
# &get_log_directory()
sub get_log_directory
{
    $_=`$exim_path -bP log_file_path 2>&1`;
    chomp;
    my @line = split_line($_);
    return pop @line
}

# Return the current version of exim
# &get_exim_version()
sub get_exim_version
{
    $_=`$exim_path -bV 2>&1`;
    return ($1, $2) if /(\d+).(\d+)/
}

# Helpers subs

sub split_line
{
    my @left_right = split /=/, $_[0], 2;
    for my $side (0..$#left_right) {
	$left_right[$side] =~ s/^\s*(.*?)\s*$/$1/
    }
    return @left_right
}

sub print_td
  {
    local $_ = shift;
    print '<td>', /^\s*$/?'&nbsp;':$_, '</td>'
  }

sub print_tr
  {
    local $_;
    print "<tr $::cb>\n";
    map { print_td ($_) } @_;
    print "<tr/>\n"
  }

sub print_th
  {
    local $_;
    print "<tr $::tb>\n";
    map { print '<th>', /^\s*$/?'&nbsp;':$_, '</th>' } @_;
    print "<tr/>\n"
  }

# Return the rank of an item in an array or undef if not found
sub rank
  {
    my ($list_to_search, $item_to_find) = @_;
    my $count = 0;

    foreach my $item (@$list_to_search) {
      last if $item eq $item_to_find;
      $count++;
    }

    return $count == @$list_to_search ? undef: $count
  }

# Perform some sanity check on CGI vars.
# Parse_CGI ('cgi', 'length', 'valid chars', 'value list')
# Example: Parse_CGI ("test\0\b\b\b", 'a-z', 'test', 'try') return 'test'
sub parse_CGI
  {
    my $cgi = shift;
    my $length = shift;
    my $valid_chars = shift;
    my @in = @_;

    my $valid_cgi = substr $cgi, 0, $length;
    eval "\$valid_cgi =~ tr/$valid_chars//c";
    if (@in)
      {
	if (grep /^$valid_cgi$/, @in)
	  { return $valid_cgi }
	else { return undef }
      }
    return $valid_cgi;
  }

# Converts base 62 ([0-9a-zA-Z]) ASCII strings to decimal numbers.
# Example: &base62_to_decimal('15gY5D');
sub  base62_to_decimal
  {
    my @tab62 =
      (0,1,2,3,4,5,6,7,8,9,0,0,0,0,0,0,      # 0-9
       0,10,11,12,13,14,15,16,17,18,19,20,   # A-K
       21,22,23,24,25,26,27,28,29,30,31,32,  # L-W
       33,34,35, 0, 0, 0, 0, 0,              # X-Z
       0,36,37,38,39,40,41,42,43,44,45,46,   # a-k
       47,48,49,50,51,52,53,54,55,56,57,58,  # l-w
       59,60,61);                            # x-z

    my($d) = 0;
    my($O) = ord('0');
    my(@c) = unpack('C*', shift);
    while($#c >= 0) { $d = $d * 62 + $tab62[shift(@c) - $O] }
    return $d;
  }

# Converts a date given as 'dd-mm-yyyy hh:mm:ss'
# in the number of seconds since the first of january 1970
sub seconds
  {
    my @days_per_month = (0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334);
    my($y,$mo,$d,$h,$mi,$s) = shift =~ /(\d+)-(\d+)-(\d+)\s(\d+):(\d+):(\d+)/;
    my($leap) = ($y % 4)? 0 : 1;
    $d += $days_per_month[$mo-1];
    $d-- unless ($leap && $mo > 2);
    $y -= 1970;
    $d += $y * 365 + ($y+1)/4;        # NB: This relies on 'use integer' pragma.
    $s += 60 * ($mi + 60 * ($h + $d * 24));
    return $s;
  }

# Replace &, >, < by their repective HTML entities '&amp;' '&gt;' '&lt;'
sub HTML_entities
  {
    local $_;
    $_ = shift;
    s/&/&amp;/;
    s/>/&gt;/;
    s/</&lt;/;
    return $_;
  }

1;
