#!/usr/bin/perl
#
#    Webmin module for Exim
#    Copyright (C) 2001 by Alexandre MATHIEU <mtlx@free.fr>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#
# spool-lib.pl

require 5;

use strict;
use integer;
use vars qw(%msg_actions %queue_actions %run_queue_by %text @time_units @size_units);
require './exim-lib.pl';

%msg_actions = (
		headers     => { cmd=>'-Mvh',
				sec=>'view' },
		body        => { cmd=>'-Mvb',
				 sec=>'view' },
		deliver     => { cmd=>'-M',
				 sec=>'proc' },
		freeze      => { cmd=>'-Mf',
				 sec=>'proc' },
		thaw        => { cmd=>'-Mt',
				 sec=>'proc' },
		give_up     => { cmd=>'-Mg',
				 sec=>'proc' },
		remove      => { cmd=>'-Mrm',
				 sec=>'proc' },
		add_rcpt    => { cmd=>'-Mar',
				 cgi=>'rcpts',
				 sec=>'mdf',
				 type=>'text' },
		edit_sender => { cmd=>'-Mes',
				 cgi=>'sender',
				 sec=>'mdf',
				 type=>'text',
				 value =>'sender' },
		mark        => { cmd=>'-Mmd',
				 cgi=>'marked',
				 sec=>'mdf',
				 type=>'select',
				 value=>'undelivered' },
		mark_all    => { cmd=>'-Mmad',
				 sec=>'mdf' },
		log         => { cmd=>'-Mvl' }
	       );

my %_messages; # Private messages store.

sub _store_messages
  {
    my $n = 0;
    my $command = '-bp ';

    if (@_) {
      for (my $i=0; $i<@_; $i++) {
	 if (exists $_messages{$_[$i]}) {
	   splice @_, $i, 1;
	   $i--;
	   $n++;
	 }
      }
      return $n unless scalar @_;
      $command .= join ' ', @_;
    }

    my $messages = &cast_exim (undef, $command);
  MESSAGE:
    while (<$messages>) {
      chomp;
      s/^\s+//; # Remove spaces at the begining of the line.

      my @fields = split /\s+/, $_; # Split the line by spaces.

      if (scalar(@fields) < 4) # If there is less than four fields...
	{
	  my ($trash, $can) = <$messages>; # throw away the result.
	  next MESSAGE
	}

      my (@delivered, @undelivered);
    RECIPIENT:
      while (<$messages>)
	{
	  chomp;
	  last RECIPIENT if /^$/; # We reach the last recipient
	  s/^\s+//;
	  if (s/^D\s//) { push @delivered, $_ } else { push @undelivered, $_ };
	}
      $fields[0] =~ /^(\d+)([mhdy])/;
      my ($time_value, $time_unit) = ($1, rank (\@time_units, $2));

      my $size_unit = $fields[1] =~ s/([KMG])$//?rank(\@size_units, $1):0;
      my $size_value = $fields[1];
      $fields[3] =~ tr/<>//d;
      $_messages{$fields[2]} = {
				time => { value => $time_value,
					  unit => $time_unit },
				size => { value => $size_value,
					  unit => $size_unit },
				sender => $fields[3],
				frozen => $fields[4]?1:0,
				undelivered => \@undelivered,
				delivered => \@delivered
			       };
      $n++;
    }

    close $messages or return undef;

    return $n
}

sub _is_message_stored
  {
    return exists $_messages{shift()}
  }

sub is_in_spool
  {
    return _store_messages (shift)
  }

sub is_frozen
  {
    my $msg_id = shift;
    _store_messages ($msg_id) or return undef;

    return $_messages{$msg_id}->{frozen}
  }

sub store_spool
  {
    return _store_messages ()
  }

sub get_spool
  {
    my $method = shift;
#    _sort_spool ($method);

    my @spool;
  SORT:
    {
      /^size/ and do {
	@spool = sort {
	  my $test = $_messages{$b}{size}{unit} <=> $_messages{$a}{size}{unit};
	  return $test?$test:$_messages{$b}{size}{value} <=> $_messages{$a}{size}{value}
	} keys %_messages;
	last SORT };
      /^time/ and do {
	@spool = sort {
	  my $test = $_messages{$b}{time}{unit} <=> $_messages{$a}{time}{unit};
	  return $test?$test:$_messages{$b}{time}{value} <=> $_messages{$a}{time}{value}
	} keys %_messages;
	last SORT };
      /^state/ and do {
	@spool = sort { $_messages{$b}{frozen} <=> $_messages{$a}{frozen} } keys %_messages;
	last SORT };
      /^sender/ and do {
	@spool = sort { $_messages{$b}{sender} <=> $_messages{$a}{sender} } keys %_messages;
	last SORT };
      @spool =  sort keys %_messages
    }
    return \@spool
  }

sub sort_spool
  {
    my ($method, @data) = @_;

    if ($method =~ /^size|time/)
      {
	my ($min, $max) = @data;
	foreach my $id (keys %_messages)
	  {
	    my $unit = $_messages{$id}{$method}{unit};
	    my $value = $_messages{$id}{$method}{value};

	    if ($unit > $$max{unit} or $unit < $$min{unit})
	      {
		delete $_messages{$id};
	      }
	    elsif ($unit == $$max{unit} and $value > $$max{value})
	      {
		delete $_messages{$id}
	      }
	    elsif ($unit == $$min{unit} and $value < $$min{value})
	      {
		delete $_messages{$id}
	      }
	  }
      }
    elsif ($method =~ /^state/)
      {
	my ($is_frozen) = @data;

	foreach my $id (keys %_messages)
	  {
	    delete $_messages{$id} unless $_messages{$id}{frozen} == $is_frozen
	  }
      }
    elsif ($method =~ /^sender/)
      {
	my ($pattern) = @data;
	foreach my $id (keys %_messages)
	  {
	    delete $_messages{$id} unless $_messages{$id}{sender} =~ /$pattern/
	  }
      }
  }

sub save_spool
  {
    use File::Basename;

    my ($tmp_file_name, $tmp_file_dir) = fileparse (tempname (), '.cgi');

    open SPOOL, '>'.$tmp_file_dir.$tmp_file_name or return undef;

    map { print SPOOL $_, "\n" } keys %_messages;

    close SPOOL or return undef;

    return $tmp_file_name;
  }

sub get_msg_info
  {
    my $msg_id = shift;
    my $property = shift;

    _store_messages ($msg_id) or return undef;

    my $msg_info =  $_messages{$msg_id}{$property};

    return $msg_info
  }

sub get_min_max
  {
    my $field = shift;

    my ($max_value, $max_unit) = (0, 0);
    my ($min_value, $min_unit);

  FIELD:
    {
      $field =~ /^time/ and do
	{
	  ($min_value, $min_unit) = (99, scalar @time_units);
	  last FIELD
	};
      $field =~ /^size/ and do
	{
	  ($min_value, $min_unit) = (9999, scalar @size_units);
	  last FIELD
	};
      return undef
    }

    foreach my $msg_id ( keys %_messages )
      {
	my $msg_unit = $_messages{$msg_id}{$field}{unit};
	my $msg_value = $_messages{$msg_id}{$field}{value};
	
	if ($msg_unit > $max_unit)
	  {
	    $max_unit = $msg_unit;
	    $max_value = $msg_value
	  }
	elsif ($msg_unit == $max_unit)
	  {
	    $max_value = $msg_value if $max_value < $msg_value
	  }
	if ($msg_unit < $min_unit)
	  {
	    $min_unit = $msg_unit;
	    $min_value = $msg_value
	  }
	elsif ($msg_unit == $min_unit)
	  {
	    $min_value = $msg_value if $min_value > $msg_value
	  }
      }
    return  {
	     min => { value => $min_value,
		      unit => $min_unit },
	     max => { value => $max_value,
		      unit => $max_unit }
	    }
  }


1;
